﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using System.IO;
using System.ComponentModel;
using System.Collections.ObjectModel;
using Infragistics.Windows.DataPresenter;

namespace XamDataGridDynamicSaveLoad
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        private ObservableCollection<Data> data = new ObservableCollection<Data>();

        public MainWindow()
        {
            this.OutputList = new ObservableCollection<String>();
            this.DataContext = this.data;

            InitializeComponent();
        }

        private void xamDataGrid1_Initialized(object sender, EventArgs e)
        {
            Field field = new Field();
            field.Name = "Open";
            xamDataGrid1.DefaultFieldLayout.Fields.Add(field);

            this.OutputList.Add("Added dynamic field");

            this.TryToLoadLayout();
        }

        private void button1_Click(object sender, RoutedEventArgs e)
        {
            try
            {
                StreamWriter tw = new StreamWriter("data.txt");
                string layout = xamDataGrid1.SaveCustomizations();
                tw.Write(layout);
                tw.Close();
            }
            catch (Exception ex)
            {
                this.OutputList.Add(String.Format("Error saving layout {1}", ex.Message));
            }
        }

        private void button2_Click(object sender, RoutedEventArgs e)
        {
            this.TryToLoadLayout();
        }

        private void TryToLoadLayout()
        {
            try
            {
                StreamReader tr = new StreamReader("data.txt");
                string layout = tr.ReadToEnd();
                xamDataGrid1.LoadCustomizations(layout);
                tr.Close();

                this.OutputList.Add("Loaded layout from data.txt.");
            }
            catch (FileNotFoundException)
            {
                this.OutputList.Add("data.txt not found, could not load layout.");
            }
            catch (Exception ex)
            {
                this.OutputList.Add(String.Format("Fehler in TryToLoadLayout(): {0}", ex.Message));
            }
        }

        private void ButtonLoadData_Click(object sender, RoutedEventArgs e)
        {
            //this.data = new ObservableCollection<Data>(Data.GetData(30, 0, 100));
            // Same behaviour with using the original collection
            var sourcedata = Data.GetData(30, 0, 100);
            foreach (var item in sourcedata)
            {
                this.data.Add(item);
            }
        }

        public ObservableCollection<string> OutputList { get; set; }
    }

    public class Data : INotifyPropertyChanged
    {
        private int open;
        private int close;
        private int high;
        private int low;
        private int value;
        private string label;
        private DateTime date;


        public DateTime Date
        {
            get { return date; }
            set
            {
                date = value;
                PropChanged("Date");
            }
        }


        public string Label
        {
            get { return label; }
            set
            {
                label = value;
                PropChanged("Label");
            }
        }

        public int Open
        {
            get { return open; }
            set
            {
                open = value;
                PropChanged("Open");
            }
        }

        public int Close
        {
            get { return close; }
            set
            {
                close = value;
                PropChanged("Close");
            }
        }

        public int High
        {
            get { return high; }
            set
            {
                high = value;
                PropChanged("High");
            }
        }

        public int Low
        {
            get { return low; }
            set
            {
                low = value;
                PropChanged("Low");
            }
        }

        public int Value
        {
            get { return value; }
            set
            {
                this.value = value;
                PropChanged("Value");
            }
        }

        public double Avarage
        {
            get
            {
                return (Open + Close + High + Low) / 4;
            }
        }

        public static ObservableCollection<Data> GetData(int points, int rangeStart, int rangeEnd)
        {
            ObservableCollection<Data> data = new ObservableCollection<Data>();
            Random r = new Random();
            for (int i = 0; i < points; i++)
            {
                data.Add(new Data
                {
                    Date = DateTime.Now.AddDays(i),
                    Label = "Item " + i,
                    Open = r.Next(rangeStart, rangeEnd),
                    Close = r.Next(rangeStart, rangeEnd),
                    High = r.Next(rangeStart, rangeEnd),
                    Low = r.Next(rangeStart, rangeEnd),
                    Value = r.Next(rangeStart, rangeEnd),
                });
            }

            return data;
        }

        public event PropertyChangedEventHandler PropertyChanged;

        public void PropChanged(string name)
        {
            if (PropertyChanged != null)
                PropertyChanged(this, new PropertyChangedEventArgs(name));
        }
    }
}
